#include <stdio.h>
#include <stdlib.h>

#define JOBQ      5         /* Number of jobs in the hold queue */
#define MAXT    100         /* Preset shortest remaining time */

/* Process Control Block */
struct Job {
        int     pid;
        int     t;
        int     cpucycle;
        char    run[256];
        int     active;
};

int
main(int argc, char **argv)
{
        /*
         * Allocate contiguous block of memory for job queue that contains
         * pointers to each PCB, and initialise a pointer to the queue.
         */
        struct Job      ***p;
        struct Job       **queue;
        int                ct;
        int                ready;
        int                shortest, time, min;
        
        queue = calloc(JOBQ, sizeof(struct Job *));
        p = &queue;
        ct = 0;
        ready = 1;
        shortest = -1;
        time = 0;
        min = MAXT;
        
        /* Allocate memory for each PCB and initialise each block to zero */
        for (int i = 0; i < JOBQ; ++i) {
                queue[i] = calloc(1, sizeof(struct Job));
        }

        /* Load data into each PCB */
        queue[0]->pid = 0;
        queue[0]->t = 0;
        queue[0]->cpucycle = 10;

        queue[1]->pid = 1;
        queue[1]->t = 1;
        queue[1]->cpucycle = 2;
        
        queue[2]->pid = 2;
        queue[2]->t = 2;
        queue[2]->cpucycle = 3;
        
        queue[3]->pid = 3;
        queue[3]->t = 3;
        queue[3]->cpucycle = 1;
        
        queue[4]->pid = 4;
        queue[4]->t = 4;
        queue[4]->cpucycle = 5;
        
        /*
         * Iterate over queue, if job has arrived, has cpu cycles, and cycles
         * remaining are fewer than current minimum——set job to shortest, and
         * job's cycle count to new minimum. Once queue iteration complete,
         * current shortest job will be set. While current shortest job has cpu
         * cycles remaining, each cycle: increment active job run counter (#);
         * increment all waiting jobs' wait counters; decrement active job's
         * cpu cycles remaining; and increment run counter pointer, and time.
         * When active job is complete, unset its active bit, and reset ready
         * status and job minimum. Continue while unfinished jobs remain. When
         * all jobs are finished, display all jobs' progression through queue.
         */
        do {
                for (int i = 0; i < JOBQ; ++i)
                        if ((*p)[i]->t <= time && (*p)[i]->cpucycle > 0 && (*p)[i]->cpucycle < min) {
                                ready = 1;
                                shortest = i;
                                min = (*p)[i]->cpucycle;
                        }

                (*p)[shortest]->active = 1;

                while ((*p)[shortest]->cpucycle > 0) {
                        (*p)[shortest]->run[ct] = '#';
                        (*p)[shortest]->cpucycle--;
                        for (int x = 0; x < JOBQ; ++x)
                                if (!(*p)[x]->active)
                                        (*p)[x]->run[ct] = '-';                                        
                        ++ct;
                        ++time;
                }
                (*p)[shortest]->active = 0;
                ready = 0;
                min = MAXT;

                for (int i = 0; i < JOBQ; ++i)
                        if ((*p)[i]->cpucycle > 0)
                                ready = 1;
        
        } while (ready == 1);

        for (int i = 0; i < JOBQ; ++i)
                printf("%s\n", (*p)[i]->run);
}